@@ -1,4 +1,4 @@
-/*	$OpenBSD: inetd.c,v 1.153 2016/03/16 20:16:39 mestre Exp $	*/
+/*	$OpenBSD: inetd.c,v 1.162 2020/12/30 18:41:06 benno Exp $	*/
 
 /*
  * Copyright (c) 1983,1991 The Regents of the University of California.
@@ -55,7 +55,7 @@
  * a space or tab.  All fields must be present in each entry.
  *
  *	service name			must be in /etc/services
- *	socket type			stream/dgram/raw/rdm/seqpacket
+ *	socket type			stream/dgram
  *	protocol			must be in /etc/protocols
  *	wait/nowait[.max]		single-threaded/multi-threaded, max #
  *	user[.group] or user[:group]	user/group to run daemon as
@@ -64,7 +64,7 @@
  *
  * For RPC services
  *      service name/version            must be in /etc/rpc
- *	socket type			stream/dgram/raw/rdm/seqpacket
+ *	socket type			stream/dgram
  *	protocol			must be in /etc/protocols
  *	wait/nowait[.max]		single-threaded/multi-threaded
  *	user[.group] or user[:group]	user to run daemon as
@@ -122,10 +122,8 @@
  */
 
 #include <sys/stat.h>
-#include <sys/ioctl.h>
 #include <sys/socket.h>
 #include <sys/un.h>
-#include <sys/file.h>
 #include <sys/wait.h>
 #include <sys/time.h>
 #include <sys/resource.h>
@@ -316,7 +314,6 @@ main(int argc, char *argv[])
 			    optarg);
 			break;
 		}
-		case '?':
 		default:
 			fprintf(stderr,
 			    "usage: inetd [-d] [-R rate] [configuration_file]\n");
@@ -358,7 +355,7 @@ main(int argc, char *argv[])
 
 	openlog("inetd", LOG_PID | LOG_NOWAIT, LOG_DAEMON);
 
-	if (getrlimit(RLIMIT_NOFILE, &rlim_nofile) < 0) {
+	if (getrlimit(RLIMIT_NOFILE, &rlim_nofile) == -1) {
 		syslog(LOG_ERR, "getrlimit: %m");
 	} else {
 		rlim_nofile_cur = rlim_nofile.rlim_cur;
@@ -401,7 +398,7 @@ gettcp(int fd, short events, void *xsep)
 	ctrl = accept(sep->se_fd, NULL, NULL);
 	if (debug)
 		fprintf(stderr, "accept, ctrl %d\n", ctrl);
-	if (ctrl < 0) {
+	if (ctrl == -1) {
 		if (errno != EWOULDBLOCK && errno != EINTR &&
 		    errno != ECONNABORTED)
 			syslog(LOG_WARNING, "accept (for %s): %m",
@@ -414,7 +411,7 @@ gettcp(int fd, short events, void *xsep)
 		socklen_t plen = sizeof(peer);
 		char sbuf[NI_MAXSERV];
 
-		if (getpeername(ctrl, (struct sockaddr *)&peer, &plen) < 0) {
+		if (getpeername(ctrl, (struct sockaddr *)&peer, &plen) == -1) {
 			syslog(LOG_WARNING, "could not getpeername");
 			close(ctrl);
 			return;
@@ -444,7 +441,6 @@ dg_badinput(struct sockaddr *sa)
 	case AF_INET:
 		in.s_addr = ntohl(((struct sockaddr_in *)sa)->sin_addr.s_addr);
 		port = ntohs(((struct sockaddr_in *)sa)->sin_port);
-	v4chk:
 		if (IN_MULTICAST(in.s_addr))
 			goto bad;
 		switch ((in.s_addr & 0xff000000) >> 24) {
@@ -460,16 +456,12 @@ dg_badinput(struct sockaddr *sa)
 		if (IN6_IS_ADDR_MULTICAST(in6) || IN6_IS_ADDR_UNSPECIFIED(in6))
 			goto bad;
 		/*
-		 * OpenBSD does not support IPv4 mapped address (RFC2553
-		 * inbound behavior) at all.  We should drop it.
+		 * OpenBSD does not support IPv4-mapped and
+		 * IPv4-compatible IPv6 addresses (RFC2553). We should
+		 * drop the packet.
 		 */
-		if (IN6_IS_ADDR_V4MAPPED(in6))
+		if (IN6_IS_ADDR_V4MAPPED(in6) || IN6_IS_ADDR_V4COMPAT(in6))
 			goto bad;
-		if (IN6_IS_ADDR_V4COMPAT(in6)) {
-			memcpy(&in, &in6->s6_addr[12], sizeof(in));
-			in.s_addr = ntohl(in.s_addr);
-			goto v4chk;
-		}
 		break;
 	default:
 		/* Unsupported AF */
@@ -491,10 +483,11 @@ dg_broadcast(struct in_addr *in)
 	struct ifaddrs *ifa, *ifap;
 	struct sockaddr_in *sin;
 
-	if (getifaddrs(&ifap) < 0)
+	if (getifaddrs(&ifap) == -1)
 		return (0);
 	for (ifa = ifap; ifa; ifa = ifa->ifa_next) {
-		if (ifa->ifa_addr->sa_family != AF_INET ||
+		if (ifa->ifa_addr == NULL ||
+		    ifa->ifa_addr->sa_family != AF_INET ||
 		    (ifa->ifa_flags & IFF_BROADCAST) == 0)
 			continue;
 		sin = (struct sockaddr_in *)ifa->ifa_broadaddr;
@@ -821,7 +814,7 @@ setup(struct servtab *sep)
 	int r;
 	mode_t mask = 0;
 
-	if ((sep->se_fd = socket(sep->se_family, sep->se_socktype, 0)) < 0) {
+	if ((sep->se_fd = socket(sep->se_family, sep->se_socktype, 0)) == -1) {
 		syslog(LOG_ERR, "%s/%s: socket: %m",
 		    sep->se_service, sep->se_proto);
 		return;
@@ -869,7 +862,7 @@ setsockopt(fd, SOL_SOCKET, opt, &on, sizeof (on))
 		if (sep->se_family == AF_UNIX)
 			umask(mask);
 	}
-	if (r < 0) {
+	if (r == -1) {
 		syslog(LOG_ERR, "%s/%s: bind: %m",
 		    sep->se_service, sep->se_proto);
 		(void) close(sep->se_fd);
@@ -913,7 +906,7 @@ register_rpc(struct servtab *sep)
 		return;
 	}
 	n = sizeof sin;
-	if (getsockname(sep->se_fd, (struct sockaddr *)&sin, &n) < 0) {
+	if (getsockname(sep->se_fd, (struct sockaddr *)&sin, &n) == -1) {
 		syslog(LOG_ERR, "%s/%s: getsockname: %m",
 		    sep->se_service, sep->se_proto);
 		return;
@@ -1103,12 +1096,6 @@ more:
 		sep->se_socktype = SOCK_STREAM;
 	else if (strcmp(arg, "dgram") == 0)
 		sep->se_socktype = SOCK_DGRAM;
-	else if (strcmp(arg, "rdm") == 0)
-		sep->se_socktype = SOCK_RDM;
-	else if (strcmp(arg, "seqpacket") == 0)
-		sep->se_socktype = SOCK_SEQPACKET;
-	else if (strcmp(arg, "raw") == 0)
-		sep->se_socktype = SOCK_RAW;
 	else
 		sep->se_socktype = -1;
 
@@ -1128,7 +1115,7 @@ more:
 
 		/* check if the family is supported */
 		s = socket(sep->se_family, SOCK_DGRAM, 0);
-		if (s < 0) {
+		if (s == -1) {
 			syslog(LOG_WARNING, "%s/%s: %s: the address family is "
 			    "not supported by the kernel", sep->se_service,
 			    sep->se_proto, sep->se_hostaddr);
@@ -1261,9 +1248,6 @@ more:
 				continue;
 			}
 			for (res = res0; res; res = res->ai_next) {
-				if (res->ai_addrlen >
-				    sizeof(sep->se_ctrladdr_storage))
-					continue;
 				/*
 				 * If sep is unused, store host in there.
 				 * Otherwise, dup a new entry and prepend it.
@@ -1428,7 +1412,7 @@ bump_nofile(void)
 
 	struct rlimit rl;
 
-	if (getrlimit(RLIMIT_NOFILE, &rl) < 0) {
+	if (getrlimit(RLIMIT_NOFILE, &rl) == -1) {
 		syslog(LOG_ERR, "getrlimit: %m");
 		return -1;
 	}
@@ -1441,7 +1425,7 @@ bump_nofile(void)
 		return -1;
 	}
 
-	if (setrlimit(RLIMIT_NOFILE, &rl) < 0) {
+	if (setrlimit(RLIMIT_NOFILE, &rl) == -1) {
 		syslog(LOG_ERR, "setrlimit: %m");
 		return -1;
 	}
@@ -1478,7 +1462,7 @@ echo_dg(int s, struct servtab *sep)
 
 	size = sizeof(ss);
 	if ((i = recvfrom(s, buffer, sizeof(buffer), 0,
-	    (struct sockaddr *)&ss, &size)) < 0)
+	    (struct sockaddr *)&ss, &size)) == -1)
 		return;
 	if (dg_badinput((struct sockaddr *)&ss))
 		return;
@@ -1569,7 +1553,7 @@ chargen_dg(int s, struct servtab *sep)
 
 	size = sizeof(ss);
 	if (recvfrom(s, text, sizeof(text), 0, (struct sockaddr *)&ss,
-	    &size) < 0)
+	    &size) == -1)
 		return;
 	if (dg_badinput((struct sockaddr *)&ss))
 		return;
@@ -1599,7 +1583,7 @@ machtime(void)
 {
 	struct timeval tv;
 
-	if (gettimeofday(&tv, NULL) < 0)
+	if (gettimeofday(&tv, NULL) == -1)
 		return (0L);
 
 	return (htonl((u_int32_t)tv.tv_sec + 2208988800UL));
@@ -1623,7 +1607,7 @@ machtime_dg(int s, struct servtab *sep)
 
 	size = sizeof(ss);
 	if (recvfrom(s, &result, sizeof(result), 0,
-	    (struct sockaddr *)&ss, &size) < 0)
+	    (struct sockaddr *)&ss, &size) == -1)
 		return;
 	if (dg_badinput((struct sockaddr *)&ss))
 		return;
@@ -1658,7 +1642,7 @@ daytime_dg(int s, struct servtab *sep)
 
 	size = sizeof(ss);
 	if (recvfrom(s, buffer, sizeof(buffer), 0, (struct sockaddr *)&ss,
-	    &size) < 0)
+	    &size) == -1)
 		return;
 	if (dg_badinput((struct sockaddr *)&ss))
 		return;
@@ -1755,7 +1739,7 @@ spawn(int ctrl, short events, void *xsep)
 		}
 		pid = fork();
 	}
-	if (pid < 0) {
+	if (pid == -1) {
 		syslog(LOG_ERR, "fork: %m");
 		if (!sep->se_wait && sep->se_socktype == SOCK_STREAM)
 			close(ctrl);
@@ -1763,17 +1747,16 @@ spawn(int ctrl, short events, void *xsep)
 		return;
 	}
 
-	if (pledge("stdio rpath getpw inet proc exec id", NULL) == -1)
-		err(1, "pledge");
-
 	if (pid && sep->se_wait) {
 		sep->se_wait = pid;
 		event_del(&sep->se_event);
 	}
 	if (pid == 0) {
-		if (sep->se_bi)
+		if (sep->se_bi) {
+			if (dofork && pledge("stdio inet", NULL) == -1)
+				err(1, "pledge");
 			(*sep->se_bi->bi_fn)(ctrl, sep);
-		else {
+		} else {
 			if ((pwd = getpwnam(sep->se_user)) == NULL) {
 				syslog(LOG_ERR,
 				    "getpwnam: %s: No such user",
@@ -1808,7 +1791,7 @@ spawn(int ctrl, short events, void *xsep)
 					tmpint |= LOGIN_SETGROUP;
 				}
 				if (setusercontext(NULL, pwd, pwd->pw_uid,
-				    tmpint) < 0) {
+				    tmpint) == -1) {
 					syslog(LOG_ERR,
 					    "%s/%s: setusercontext: %m",
 					    sep->se_service, sep->se_proto);
